"use strict";
var factory_1 = require("../core/factory");
var utils_1 = require("../core/utils");
var core_1 = require("../core");
var registrar_1 = require("../router/registrar");
var interfaces_1 = require("../core/interfaces");
var context_1 = require("../core/context");
var configuration_1 = require("../config/configuration");
var logger_1 = require("../adapter/logger");
var fs = require("fs");
var path = require("path");
var WebServer = (function () {
    function WebServer() {
        this.logger = new logger_1.Adapter({ bufferLogs: true });
        this.tasks = [];
        this.logger.log("Application is starting...", typeof (this));
        this.execute();
    }
    WebServer.start = function () {
        if (WebServer._instance == null) {
            WebServer._instance = factory_1.Container.get(WebServer);
        }
    };
    Object.defineProperty(WebServer.prototype, "context", {
        get: function () {
            return this._context;
        },
        enumerable: true,
        configurable: true
    });
    WebServer.prototype.execute = function () {
        this.initializeSystem();
        this.initializeApplication();
    };
    WebServer.prototype.initializeSystem = function () {
        this.initializeConfiguration();
        this.initializeAdapter();
        this.initializeContext();
    };
    WebServer.prototype.initializeApplication = function () {
        var _this = this;
        this.initializeController();
        this.initializeRequestHandler();
        this.initializeTasks();
        this.initializeRoutes();
        this.executeTasks()
            .then(function () { _this.startWebServer(); });
    };
    WebServer.prototype.initializeConfiguration = function () {
        this.config = factory_1.Container.get(configuration_1.Configuration);
    };
    WebServer.prototype.initializeAdapter = function () {
        this.registerAdapter(interfaces_1.ILogger, "logger");
        this.registerAdapter(interfaces_1.IFramework, "framework");
        this.registerAdapter(interfaces_1.IDataAdapter, "data");
    };
    WebServer.prototype.registerAdapter = function (typeOfAdapter, name) {
        var modulePath = this.resolveModulePath("/adapter", name);
        var module = require(modulePath[0]);
        factory_1.Container.bind(typeOfAdapter).to(module.Adapter);
    };
    WebServer.prototype.initializeController = function () {
        var pathToControllers = "/controller";
        this.resolveModules(pathToControllers);
    };
    WebServer.prototype.initializeRequestHandler = function () {
        var pathToControllers = "/handler";
        this.resolveModules(pathToControllers);
    };
    WebServer.prototype.resolveModules = function (pathFragment) {
        var _this = this;
        var modules = [];
        var fullModulePaths = this.resolveModulePath(pathFragment, "");
        fullModulePaths.forEach(function (fullModulePath) {
            if (utils_1.FileSystemHelper.isFolder(fullModulePath)) {
                var filenamesInFolder = fs.readdirSync(fullModulePath);
                filenamesInFolder.forEach(function (filename) {
                    if (filename.match(/[^\.]*\.js$/i)) {
                        var fullFilePath = path.posix.resolve(fullModulePath, filename);
                        _this.logger.log(fullFilePath);
                        var module_1 = require(fullFilePath);
                        _this.logger.log(module_1);
                        modules.push(module_1);
                    }
                });
            }
            else {
                throw new Error("Modules could not be resolved by path. Path does not exists [" + pathFragment + "].");
            }
        });
        return modules;
    };
    WebServer.prototype.resolveModulePath = function (path, filename) {
        var paths = [];
        var filePath = (filename !== "") ? "/" + filename : "";
        if (path.indexOf(this.config.settings.basePath) === -1) {
            var foldersToSearch = [];
            foldersToSearch.push(this.config.coreBasePath + "/system/" + path);
            foldersToSearch.push(this.config.coreBasePath + "/" + path);
            foldersToSearch.push(this.config.appBasePath + "/system/" + path);
            foldersToSearch.push(this.config.appBasePath + "/" + path);
            var newPath = "";
            while (foldersToSearch.length !== 0) {
                newPath = foldersToSearch.pop();
                if (utils_1.FileSystemHelper.fileOrFolderExists(newPath + filePath)) {
                    path = newPath;
                    var absoluteFilePath = path + filePath;
                    paths.push(fs.realpathSync(absoluteFilePath));
                }
            }
            if (paths.length === 0) {
                throw new Error("Path could not be determined. Path [" + path + "], File [" + filename + "].");
            }
        }
        return paths;
    };
    WebServer.prototype.initializeTasks = function () {
        var _this = this;
        var pathToInitializers = "/initializers";
        this.resolveModules(pathToInitializers).forEach(function (module) {
            for (var key in module) {
                if (module.hasOwnProperty(key)) {
                    _this.tasks.push({ name: key, instance: module[key] });
                }
            }
        });
    };
    WebServer.prototype.initializeContext = function () {
        this._context = factory_1.Container.get(context_1.Context);
        this.switchLogger();
    };
    WebServer.prototype.switchLogger = function () {
        var _this = this;
        var logger = this.logger;
        this.logger = this.context.logger;
        logger.flushBuffer(function (logData) {
            _this.logger.log(logData.message, logData.optionalParams);
        });
    };
    WebServer.prototype.initializeRoutes = function () {
        var registrar = factory_1.Container.get(registrar_1.RouteRegistrar);
        registrar.registerHttpHandler("/1");
        registrar.registerRoutes();
        registrar.registerHttpHandler("/2");
    };
    WebServer.prototype.executeTasks = function () {
        var _this = this;
        var promises = [];
        this.tasks.forEach(function (task) {
            var promise = factory_1.Container.get(task.instance).execute();
            promise
                .then(function (data) { _this.onTaskSuccess({ name: task.name, data: data }); })
                .catch(function (error) { _this.onTaskError({ name: task.name, error: error }); });
            promises.push(promise);
        });
        return core_1.Promise.all(promises);
    };
    WebServer.prototype.onTaskSuccess = function (task) {
        this.logger.log("Task executed", task.name);
    };
    WebServer.prototype.onTaskError = function (task) {
        this.logger.log("Task failed", task.name, task.error);
        throw task.error;
    };
    WebServer.prototype.onBeforeServerStart = function () {
    };
    ;
    WebServer.prototype.onAfterServerStart = function () {
    };
    ;
    WebServer.prototype.startWebServer = function () {
        var _this = this;
        this.onBeforeServerStart();
        this.context.framework.startWebServer(this.context.settings.port, this.context.settings.hostname, function () { _this.onServerStarted(); });
    };
    WebServer.prototype.onServerStarted = function () {
        var url = this.context.settings.protocol
            + "://" + this.context.settings.hostname
            + ":" + this.context.settings.port
            + this.context.settings.root;
        this.logger.log("Application Web Server is running at", url);
        this.onAfterServerStart();
    };
    return WebServer;
}());
exports.WebServer = WebServer;

//# sourceMappingURL=../../../maps/src/system/core/webserver.js.map
